"""
Torch run example: python examples/text_to_image_deep_cache_sd.py --compile 0
Compile to oneflow graph example: python examples/text_to_image_deep_cache_sd.py
"""
import argparse
import os

import torch

from onediff.infer_compiler import oneflow_compile
from onediff.schedulers import EulerDiscreteScheduler

from onediffx.deep_cache import StableDiffusionPipeline

parser = argparse.ArgumentParser()
parser.add_argument("--base", type=str, default="runwayml/stable-diffusion-v1-5")
parser.add_argument("--variant", type=str, default="fp16")
parser.add_argument(
    "--prompt",
    type=str,
    default="street style, detailed, raw photo, woman, face, shot on CineStill 800T",
)
parser.add_argument("--height", type=int, default=1024)
parser.add_argument("--width", type=int, default=1024)
parser.add_argument("--n_steps", type=int, default=30)
parser.add_argument("--saved_image", type=str, required=False, default="sd-out.png")
parser.add_argument("--warmup", type=int, default=1)
parser.add_argument("--seed", type=int, default=1)
parser.add_argument(
    "--compile",
    type=(lambda x: str(x).lower() in ["true", "1", "yes"]),
    default=True,
)
parser.add_argument(
    "--use_multiple_resolutions",
    type=(lambda x: str(x).lower() in ["true", "1", "yes"]),
    default=False,
)
args = parser.parse_args()

# Normal SD-1.5 pipeline init.
OUTPUT_TYPE = "pil"

# SD-1.5 base: StableDiffusionPipeline
scheduler = EulerDiscreteScheduler.from_pretrained(args.base, subfolder="scheduler")
base = StableDiffusionPipeline.from_pretrained(
    args.base,
    scheduler=scheduler,
    torch_dtype=torch.float16,
    variant=args.variant,
    use_safetensors=True,
)
base.to("cuda")

# Compile unet with oneflow
if args.compile:
    print("Compiling unet with oneflow.")
    base.unet = oneflow_compile(base.unet)
    base.fast_unet = oneflow_compile(base.fast_unet)
    base.vae.decoder = oneflow_compile(base.vae.decoder)


# Define multiple resolutions for warmup
resolutions = (
    [
        (512, 512),
        (256, 256),
    ]
    if args.use_multiple_resolutions
    else [(args.height, args.width)]
)

# Warmup with chosen resolutions
for resolution in resolutions:
    for i in range(args.warmup):
        torch.manual_seed(args.seed)
        image = base(
            prompt=args.prompt,
            height=resolution[0],
            width=resolution[1],
            num_inference_steps=args.n_steps,
            output_type=OUTPUT_TYPE,
            cache_interval=5,
            cache_layer_id=0,
            cache_block_id=0,
            uniform=False,
            pow=1.4,
            center=15,
        ).images

# Normal SD-1.5 run
torch.manual_seed(args.seed)
image = base(
    prompt=args.prompt,
    height=args.height,
    width=args.width,
    num_inference_steps=args.n_steps,
    output_type=OUTPUT_TYPE,
    cache_interval=5,
    cache_layer_id=0,
    cache_block_id=0,
    uniform=False,
    pow=1.4,
    center=15,
).images
image[0].save(f"h{args.height}-w{args.width}-{args.saved_image}")



import argparse
import os
import time

import torch
from diffusers import StableDiffusionXLPipeline
from huggingface_hub import hf_hub_download
from onediffx import compile_pipe, load_pipe, save_pipe
from safetensors.torch import load_file

try:
    USE_PEFT_BACKEND = diffusers.utils.USE_PEFT_BACKEND
except Exception as e:
    USE_PEFT_BACKEND = False

parser = argparse.ArgumentParser()
parser.add_argument(
    "--base", type=str, default="stabilityai/stable-diffusion-xl-base-1.0"
)
parser.add_argument("--repo", type=str, default="ByteDance/SDXL-Lightning")
parser.add_argument("--cpkt", type=str, default="sdxl_lightning_4step_unet.safetensors")
parser.add_argument("--variant", type=str, default="fp16")
parser.add_argument(
    "--prompt",
    type=str,
    # default="street style, detailed, raw photo, woman, face, shot on CineStill 800T",
    default="A girl smiling",
)
parser.add_argument("--save_graph", action="store_true")
parser.add_argument("--load_graph", action="store_true")
parser.add_argument("--save_graph_dir", type=str, default="cached_pipe")
parser.add_argument("--load_graph_dir", type=str, default="cached_pipe")
parser.add_argument("--height", type=int, default=1024)
parser.add_argument("--width", type=int, default=1024)
parser.add_argument(
    "--saved_image", type=str, required=False, default="sdxl-light-out.png"
)
parser.add_argument("--seed", type=int, default=1)
parser.add_argument(
    "--compile",
    type=(lambda x: str(x).lower() in ["true", "1", "yes"]),
    default=True,
)


args = parser.parse_args()

OUTPUT_TYPE = "pil"

n_steps = int(args.cpkt[len("sdxl_lightning_") : len("sdxl_lightning_") + 1])

is_lora_cpkt = "lora" in args.cpkt

if args.compile:
    from onediff.schedulers import EulerDiscreteScheduler
else:
    from diffusers import EulerDiscreteScheduler

if is_lora_cpkt:
    if not USE_PEFT_BACKEND:
        print("PEFT backend is required for load_lora_weights")
        exit(0)
    pipe = StableDiffusionXLPipeline.from_pretrained(
        args.base, torch_dtype=torch.float16, variant="fp16"
    ).to("cuda")
    if os.path.isfile(os.path.join(args.repo, args.cpkt)):
        pipe.load_lora_weights(os.path.join(args.repo, args.cpkt))
    else:
        pipe.load_lora_weights(hf_hub_download(args.repo, args.cpkt))
    pipe.fuse_lora()
else:
    from diffusers import UNet2DConditionModel

    unet = UNet2DConditionModel.from_config(args.base, subfolder="unet").to(
        "cuda", torch.float16
    )
    if os.path.isfile(os.path.join(args.repo, args.cpkt)):
        unet.load_state_dict(
            load_file(os.path.join(args.repo, args.cpkt), device="cuda")
        )
    else:
        unet.load_state_dict(
            load_file(hf_hub_download(args.repo, args.cpkt), device="cuda")
        )
    pipe = StableDiffusionXLPipeline.from_pretrained(
        args.base, unet=unet, torch_dtype=torch.float16, variant="fp16"
    ).to("cuda")

pipe.scheduler = EulerDiscreteScheduler.from_config(
    pipe.scheduler.config, timestep_spacing="trailing"
)

if pipe.vae.dtype == torch.float16 and pipe.vae.config.force_upcast:
    pipe.upcast_vae()

# Compile the pipeline
if args.compile:
    pipe = compile_pipe(
        pipe,
    )
    if args.load_graph:
        print("Loading graphs...")
        load_pipe(pipe, args.load_graph_dir)

print("Warmup with running graphs...")
torch.manual_seed(args.seed)
image = pipe(
    prompt=args.prompt,
    height=args.height,
    width=args.width,
    num_inference_steps=n_steps,
    guidance_scale=0,
    output_type=OUTPUT_TYPE,
).images


# Normal run
print("Normal run...")
torch.manual_seed(args.seed)
start_t = time.time()
image = pipe(
    prompt=args.prompt,
    height=args.height,
    width=args.width,
    num_inference_steps=n_steps,
    guidance_scale=0,
    output_type=OUTPUT_TYPE,
).images

end_t = time.time()
print(f"e2e ({n_steps} steps) elapsed: {end_t - start_t} s")

image[0].save(args.saved_image)

if args.save_graph:
    print("Saving graphs...")
    save_pipe(pipe, args.save_graph_dir)



import argparse
import importlib.metadata
import os

from packaging import version


def parse_args():
    parser = argparse.ArgumentParser(description="Simple demo of LCM image generation.")
    parser.add_argument(
        "--prompt",
        type=str,
        default="Self-portrait oil painting, a beautiful cyborg with golden hair, 8k",
    )
    parser.add_argument(
        "--model_id",
        type=str,
        default="stabilityai/stable-diffusion-xl-base-1.0",
        help="Model id or local path to the Stable Diffusion base model.",
    )
    parser.add_argument(
        "--adapter_id",
        type=str,
        default="latent-consistency/lcm-lora-sdxl",
        help="Model id or local path to the LCM Lora adapter model.",
    )
    parser.add_argument("--height", type=int, default=1024)
    parser.add_argument("--width", type=int, default=1024)
    parser.add_argument("--steps", type=int, default=4)
    parser.add_argument(
        "--warmup",
        type=int,
        default=1,
        help="Warmup option sets the number of preliminary runs of the pipeline. These initial runs are necessary to ensure accurate performance data during testing.",
    )
    parser.add_argument("--seed", type=int, default=1)
    parser.add_argument(
        "--disable", action="store_true", help="Disable Onediff speeding up."
    )

    parser.add_argument(
        "--mlir_enable_inference_optimization",
        action="store_true",
        help="If this option is enabled, it will trigger additional optimizations, but it may potentially affect the quality of the output image.",
    )
    parser.add_argument(
        "--output", type=str, default="", help="Output image file name."
    )

    args = parser.parse_args()
    return args


from diffusers import AutoPipelineForText2Image, LCMScheduler

args = parse_args()

if not args.disable:
    from onediff.infer_compiler import oneflow_compile
import torch

pipe = AutoPipelineForText2Image.from_pretrained(
    args.model_id, torch_dtype=torch.float16, variant="fp16"
)
pipe.scheduler = LCMScheduler.from_config(pipe.scheduler.config)
pipe.to("cuda")

if not args.disable:
    pipe.unet = oneflow_compile(pipe.unet)


if not args.mlir_enable_inference_optimization:
    os.environ["ONEFLOW_MLIR_ENABLE_INFERENCE_OPTIMIZATION"] = "0"
else:
    os.environ["ONEFLOW_MLIR_ENABLE_INFERENCE_OPTIMIZATION"] = "1"

pipe.load_lora_weights(args.adapter_id)
if hasattr(pipe, "fuse_lora"):
    pipe.fuse_lora()
else:
    print(f"pipe.fuse_lora doesn't exist, exit")
    exit(1)

for _ in range(args.warmup):
    torch.manual_seed(args.seed)
    images = pipe(
        args.prompt,
        height=args.height,
        width=args.width,
        num_inference_steps=args.steps,
    ).images

torch.manual_seed(args.seed)
images = pipe(
    args.prompt, height=args.height, width=args.width, num_inference_steps=args.steps
).images
for i, image in enumerate(images):
    if args.output == "":
        image.save(
            f"LCM-LoRA-{args.width}x{args.height}-seed-{args.seed}-disable-{args.disable}-mlir-{args.mlir_enable_inference_optimization}.png"
        )
    else:
        image.save(f"{args.output}.png")



# Compile and save to oneflow graph example: python examples/text_to_image_sdxl_save_load.py --save
# Compile and load to oneflow graph example: python examples/text_to_image_sdxl_save_load.py --load

import argparse
import os

import torch
import oneflow as flow  # usort: skip

from diffusers import DiffusionPipeline
from onediff.infer_compiler import oneflow_compile, OneflowCompileOptions

parser = argparse.ArgumentParser()
parser.add_argument(
    "--base", type=str, default="stabilityai/stable-diffusion-xl-base-1.0"
)
parser.add_argument("--variant", type=str, default="fp16")
parser.add_argument(
    "--prompt",
    type=str,
    default="street style, detailed, raw photo, woman, face, shot on CineStill 800T",
)
parser.add_argument("--n_steps", type=int, default=3)
parser.add_argument("--saved_image", type=str, required=False, default="sdxl-out.png")
parser.add_argument("--seed", type=int, default=1)
parser.add_argument("--num_dynamic_input_size", type=int, default=9)
parser.add_argument("--save", action=argparse.BooleanOptionalAction)
parser.add_argument("--load", action=argparse.BooleanOptionalAction)
cmd_args = parser.parse_args()


# Normal SDXL pipeline init.
SEED = torch.Generator("cuda").manual_seed(cmd_args.seed)
OUTPUT_TYPE = "pil"
# SDXL base: StableDiffusionXLPipeline
base = DiffusionPipeline.from_pretrained(
    cmd_args.base,
    torch_dtype=torch.float16,
    variant=cmd_args.variant,
    use_safetensors=True,
)
base.to("cuda")

# To fix the bug of graph load of vae. Please refer to: https://github.com/siliconflow/onediff/issues/452
if base.vae.dtype == torch.float16 and base.vae.config.force_upcast:
    base.upcast_vae()

# Compile unet and vae
print("unet and vae is compiled to oneflow.")
compile_options = OneflowCompileOptions()
compile_options.max_cached_graph_size = cmd_args.num_dynamic_input_size

base.unet = oneflow_compile(base.unet, options=compile_options)
base.vae.decoder = oneflow_compile(base.vae.decoder, options=compile_options)
if base.text_encoder is not None:
    base.text_encoder = oneflow_compile(base.text_encoder, options=compile_options)
if base.text_encoder_2 is not None:
    base.text_encoder_2 = oneflow_compile(base.text_encoder_2, options=compile_options)

if cmd_args.load:
    print("Loading graphs to avoid compilation...")
    # run_warmup is True to run unet/vae once to make the cuda runtime ready.
    base.unet.load_graph("base_unet_compiled", run_warmup=True)
    base.vae.decoder.load_graph("base_vae_compiled", run_warmup=True)
    if base.text_encoder is not None:
        print("Loading text_encoder...")
        base.text_encoder.load_graph("base_text_encoder_compiled", run_warmup=True)
    if base.text_encoder_2 is not None:
        print("Loading text_encoder_2...")
        base.text_encoder_2.load_graph("base_text_encoder_2_compiled", run_warmup=True)
else:
    print("Warmup with running graphs...")
    sizes = [1024, 896]
    for h in sizes:
        for w in sizes:
            for i in range(2):
                image = base(
                    prompt=cmd_args.prompt,
                    height=h,
                    width=w,
                    generator=SEED,
                    num_inference_steps=cmd_args.n_steps,
                    output_type=OUTPUT_TYPE,
                ).images


# Normal SDXL run
print("Normal SDXL run...")
sizes = [1024, 896]
for h in sizes:
    for w in sizes:
        for i in range(2):
            image = base(
                prompt=cmd_args.prompt,
                height=h,
                width=w,
                generator=SEED,
                num_inference_steps=cmd_args.n_steps,
                output_type=OUTPUT_TYPE,
            ).images
            image[0].save(f"h{h}-w{w}-i{i}-{cmd_args.saved_image}")

# Save compiled graphs with oneflow
if cmd_args.save:
    print("Saving graphs...")
    base.unet.save_graph("base_unet_compiled")
    base.vae.decoder.save_graph("base_vae_compiled")
    if base.text_encoder is not None:
        base.text_encoder.save_graph("base_text_encoder_compiled")
    if base.text_encoder_2 is not None:
        base.text_encoder_2.save_graph("base_text_encoder_2_compiled")



"""
Torch run example: python examples/text_to_image_sdxl.py
Compile with oneflow: python examples/text_to_image_sdxl.py --compiler oneflow
Compile with nexfort: python examples/text_to_image_sdxl.py --compiler nexfort
Test dynamic shape: Add --run_multiple_resolutions 1 and --run_rare_resolutions 1
"""

import argparse
import json
import os
import time

import torch
import oneflow as flow  # usort: skip

from diffusers import StableDiffusionXLPipeline

# from onediff.infer_compiler import oneflow_compile
from onediff.schedulers import EulerDiscreteScheduler
from onediffx import compile_pipe

parser = argparse.ArgumentParser()
parser.add_argument(
    "--base", type=str, default="stabilityai/stable-diffusion-xl-base-1.0"
)
parser.add_argument("--variant", type=str, default="fp16")
parser.add_argument(
    "--prompt",
    type=str,
    default="street style, detailed, raw photo, woman, face, shot on CineStill 800T",
)
parser.add_argument("--height", type=int, default=1024)
parser.add_argument("--width", type=int, default=1024)
parser.add_argument("--n_steps", type=int, default=30)
parser.add_argument("--saved_image", type=str, required=False, default="sdxl-out.png")
parser.add_argument("--seed", type=int, default=1)
# parser.add_argument(
#     "--compile_unet",
#     type=(lambda x: str(x).lower() in ["true", "1", "yes"]),
#     default=True,
# )
# parser.add_argument(
#     "--compile_vae",
#     type=(lambda x: str(x).lower() in ["true", "1", "yes"]),
#     default=True,
# )
parser.add_argument(
    "--compiler",
    type=str,
    default="oneflow",
    choices=["oneflow", "nexfort"],
)
parser.add_argument(
    "--compiler-config",
    type=str,
    default=None,
)
parser.add_argument(
    "--run_multiple_resolutions",
    type=(lambda x: str(x).lower() in ["true", "1", "yes"]),
    default=True,
)
parser.add_argument(
    "--run_rare_resolutions",
    type=(lambda x: str(x).lower() in ["true", "1", "yes"]),
    default=True,
)
args = parser.parse_args()

# Normal SDXL pipeline init.
OUTPUT_TYPE = "pil"

# SDXL base: StableDiffusionXLPipeline
scheduler = EulerDiscreteScheduler.from_pretrained(args.base, subfolder="scheduler")
base = StableDiffusionXLPipeline.from_pretrained(
    args.base,
    scheduler=scheduler,
    torch_dtype=torch.float16,
    variant=args.variant,
    use_safetensors=True,
)
base.to("cuda")

# # Compile unet with oneflow
# if args.compile_unet:
#     print("Compiling unet with oneflow.")
#     base.unet = oneflow_compile(base.unet)

# # Compile vae with oneflow
# if args.compile_vae:
#     print("Compiling vae with oneflow.")
#     base.vae.decoder = oneflow_compile(base.vae.decoder)

# Compile the pipe
if args.compiler == "oneflow":
    base = compile_pipe(base)
elif args.compiler == "nexfort":
    if args.compiler_config is not None:
        options = json.loads(args.compiler_config)
    else:
        options = json.loads('{"mode": "max-autotune:cudagraphs", "dynamic": true}')

    base = compile_pipe(
        base, backend="nexfort", options=options, fuse_qkv_projections=True
    )

# Warmup with run
# Will do compilatioin in the first run
print("Warmup with running graphs...")
torch.manual_seed(args.seed)
image = base(
    prompt=args.prompt,
    height=args.height,
    width=args.width,
    num_inference_steps=args.n_steps,
    output_type=OUTPUT_TYPE,
).images

# Normal SDXL run
print("Normal SDXL run...")
torch.manual_seed(args.seed)
print(f"Running at resolution: {args.height}x{args.width}")
start_time = time.time()
image = base(
    prompt=args.prompt,
    height=args.height,
    width=args.width,
    num_inference_steps=args.n_steps,
    output_type=OUTPUT_TYPE,
).images
end_time = time.time()
print(f"Inference time: {end_time - start_time:.2f} seconds")
image[0].save(f"h{args.height}-w{args.width}-{args.saved_image}")


# Should have no compilation for these new input shape
# The nexfort backend encounters an exception when dynamically switching resolution to 960x720.
if args.run_multiple_resolutions:
    print("Test run with multiple resolutions...")
    sizes = [960, 720, 896, 768]
    if "CI" in os.environ:
        sizes = [360]
    for h in sizes:
        for w in sizes:
            print(f"Running at resolution: {h}x{w}")
            start_time = time.time()
            image = base(
                prompt=args.prompt,
                height=h,
                width=w,
                num_inference_steps=args.n_steps,
                output_type=OUTPUT_TYPE,
            ).images
            end_time = time.time()
            print(f"Inference time: {end_time - start_time:.2f} seconds")


if args.run_rare_resolutions:
    print("Test run with other another uncommon resolution...")
    h = 544
    w = 408
    print(f"Running at resolution: {h}x{w}")
    start_time = time.time()
    image = base(
        prompt=args.prompt,
        height=h,
        width=w,
        num_inference_steps=args.n_steps,
        output_type=OUTPUT_TYPE,
    ).images
    end_time = time.time()
    print(f"Inference time: {end_time - start_time:.2f} seconds")



import argparse

import torch
from PIL import Image
import oneflow as flow  # usort: skip

from diffusers import StableDiffusionImg2ImgPipeline
from onediff.infer_compiler import oneflow_compile


prompt = "sea,beach,the waves crashed on the sand,blue sky whit white cloud"


def parse_args():
    parser = argparse.ArgumentParser(description="Simple demo of image generation.")
    parser.add_argument(
        "--model_id",
        type=str,
        default="stabilityai/stable-diffusion-2-1",
    )
    cmd_args = parser.parse_args()
    return cmd_args


args = parse_args()


pipe = StableDiffusionImg2ImgPipeline.from_pretrained(
    args.model_id,
    use_auth_token=True,
    revision="fp16",
    torch_dtype=torch.float16,
)

pipe = pipe.to("cuda")
pipe.unet = oneflow_compile(pipe.unet)
pipe.vae.decoder = oneflow_compile(pipe.vae.decoder)


img = Image.new("RGB", (512, 512), "#1f80f0")

with flow.autocast("cuda"):
    images = pipe(
        prompt,
        image=img,
        guidance_scale=10,
        num_inference_steps=100,
        output_type="np",
    ).images
    for i, image in enumerate(images):
        pipe.numpy_to_pil(image)[0].save(f"{prompt}-of-{i}.png")



# Command to run save: python test_pipe_compile_save_load.py --save
# Command to run load: python test_pipe_compile_save_load.py --load
import argparse

import torch
from diffusers import StableDiffusionXLPipeline
from onediffx import compile_pipe, load_pipe, save_pipe

parser = argparse.ArgumentParser()
parser.add_argument(
    "--model", type=str, default="stabilityai/stable-diffusion-xl-base-1.0"
)
parser.add_argument("--save", action=argparse.BooleanOptionalAction)
parser.add_argument("--load", action=argparse.BooleanOptionalAction)
cmd_args = parser.parse_args()

pipe = StableDiffusionXLPipeline.from_pretrained(
    "/share_nfs/hf_models/stable-diffusion-xl-base-1.0",
    torch_dtype=torch.float16,
    variant="fp16",
    use_safetensors=True,
)
pipe.to("cuda")

# compile the pipe
pipe = compile_pipe(pipe)

if cmd_args.load:
    # load the compiled pipe
    load_pipe(pipe, dir="cached_pipe")

# If the pipe is not loaded, it will takes seconds to do real compilation.
# If the pipe is loaded, it will run fast.
image = pipe(
    prompt="street style, detailed, raw photo, woman, face, shot on CineStill 800T",
    height=512,
    width=512,
    num_inference_steps=30,
    output_type="pil",
).images

image[0].save(f"test_image.png")

if cmd_args.save:
    # save the compiled pipe
    save_pipe(pipe, dir="cached_pipe")



import argparse
import os

import torch
from diffusers import StableDiffusionXLPipeline

from onediff.infer_compiler import oneflow_compile

# import diffusers
# diffusers.logging.set_verbosity_info()

parser = argparse.ArgumentParser()
parser.add_argument(
    "--base", type=str, default="stabilityai/stable-diffusion-xl-base-1.0"
)
parser.add_argument(
    "--new_base",
    type=str,
    default="dataautogpt3/OpenDalleV1.1",
)
parser.add_argument("--variant", type=str, default="fp16")
parser.add_argument(
    "--prompt",
    type=str,
    default="street style, detailed, raw photo, woman, face, shot on CineStill 800T",
)
parser.add_argument("--height", type=int, default=1024)
parser.add_argument("--width", type=int, default=1024)
parser.add_argument("--n_steps", type=int, default=30)
parser.add_argument("--guidance_scale", type=float, default=7.5)
parser.add_argument("--saved_image", type=str, required=False, default="sdxl-out.png")
parser.add_argument("--seed", type=int, default=1)
parser.add_argument(
    "--compile_unet",
    type=(lambda x: str(x).lower() in ["true", "1", "yes"]),
    default=True,
)
parser.add_argument(
    "--compile_vae",
    type=(lambda x: str(x).lower() in ["true", "1", "yes"]),
    default=True,
)
parser.add_argument(
    "--run_multiple_resolutions",
    type=(lambda x: str(x).lower() in ["true", "1", "yes"]),
    default=True,
)
args = parser.parse_args()

# Normal SDXL pipeline init.
OUTPUT_TYPE = "pil"

# SDXL base: StableDiffusionXLPipeline
base = StableDiffusionXLPipeline.from_pretrained(
    args.base,
    torch_dtype=torch.float16,
    variant=args.variant,
    use_safetensors=True,
)
base.to("cuda")

# Compile unet with oneflow
if args.compile_unet:
    print("Compiling unet with oneflow.")
    compiled_unet = oneflow_compile(base.unet)
    base.unet = compiled_unet

# Compile vae with oneflow
if args.compile_vae:
    print("Compiling vae with oneflow.")
    compiled_decoder = oneflow_compile(base.vae.decoder)
    base.vae.decoder = compiled_decoder

# Warmup with run
# Will do compilatioin in the first run
print("Warmup with running graphs...")
torch.manual_seed(args.seed)
image = base(
    prompt=args.prompt,
    height=args.height,
    width=args.width,
    num_inference_steps=args.n_steps,
    generator=torch.manual_seed(0),
    output_type=OUTPUT_TYPE,
    guidance_scale=args.guidance_scale,
).images
del base

torch.cuda.empty_cache()

print("loading new base")
if str(args.new_base).endswith(".safetensors"):
    new_base = StableDiffusionXLPipeline.from_single_file(
        args.new_base,
        torch_dtype=torch.float16,
        variant=args.variant,
        use_safetensors=True,
    )
else:
    new_base = StableDiffusionXLPipeline.from_pretrained(
        args.new_base,
        torch_dtype=torch.float16,
        variant=args.variant,
        use_safetensors=True,
    )
new_base.to("cuda")

print("New base running by torch backend")
torch.manual_seed(args.seed)
image = new_base(
    prompt=args.prompt,
    height=args.height,
    width=args.width,
    num_inference_steps=args.n_steps,
    generator=torch.manual_seed(0),
    output_type=OUTPUT_TYPE,
    guidance_scale=args.guidance_scale,
).images
image[0].save(f"new_base_without_graph_h{args.height}-w{args.width}-{args.saved_image}")
image_eager = image[0]


# Update the unet and vae
# load_state_dict(state_dict, strict=True, assign=False), assign is False means copying them inplace into the module’s current parameters and buffers.
# Reference: https://pytorch.org/docs/stable/generated/torch.nn.Module.html#torch.nn.Module.load_state_dict
print("Loading state_dict of new base into compiled graph")
compiled_unet._torch_module.load_state_dict(new_base.unet.state_dict())
compiled_decoder._torch_module.load_state_dict(new_base.vae.decoder.state_dict())

new_base.unet = compiled_unet
new_base.vae.decoder = compiled_decoder

torch.cuda.empty_cache()

# Normal SDXL run
print("Re-use the compiled graph")
torch.manual_seed(args.seed)
image = new_base(
    prompt=args.prompt,
    height=args.height,
    width=args.width,
    num_inference_steps=args.n_steps,
    generator=torch.manual_seed(0),
    output_type=OUTPUT_TYPE,
    guidance_scale=args.guidance_scale,
).images
image[0].save(f"new_base_reuse_graph_h{args.height}-w{args.width}-{args.saved_image}")
image_graph = image[0]

import numpy as np
from skimage.metrics import structural_similarity

ssim = structural_similarity(
    np.array(image_eager), np.array(image_graph), channel_axis=-1, data_range=255
)
print(f"ssim between naive torch and re-used graph is {ssim}")


# Should have no compilation for these new input shape
print("Test run with multiple resolutions...")
if args.run_multiple_resolutions:
    sizes = [960, 720, 896, 768]
    if "CI" in os.environ:
        sizes = [360]
    for h in sizes:
        for w in sizes:
            image = new_base(
                prompt=args.prompt,
                height=h,
                width=w,
                num_inference_steps=args.n_steps,
                generator=torch.manual_seed(0),
                output_type=OUTPUT_TYPE,
            ).images


# print("Test run with other another uncommon resolution...")
# if args.run_multiple_resolutions:
#     h = 544
#     w = 408
#     image = base(
#         prompt=args.prompt,
#         height=h,
#         width=w,
#         num_inference_steps=args.n_steps,
#         output_type=OUTPUT_TYPE,
#     ).images



"""
example: python examples/text_to_image.py --height 512 --width 512 --warmup 10 --model_id xx
"""
import argparse

import torch
import oneflow as flow  # usort: skip

from diffusers import StableDiffusionPipeline
from onediff.infer_compiler import oneflow_compile
from onediff.schedulers import EulerDiscreteScheduler


def parse_args():
    parser = argparse.ArgumentParser(description="Simple demo of image generation.")
    parser.add_argument(
        "--prompt", type=str, default="a photo of an astronaut riding a horse on mars"
    )
    parser.add_argument(
        "--model_id",
        type=str,
        default="runwayml/stable-diffusion-v1-5",
    )
    parser.add_argument("--height", type=int, default=512)
    parser.add_argument("--width", type=int, default=512)
    parser.add_argument("--steps", type=int, default=30)
    parser.add_argument("--warmup", type=int, default=1)
    parser.add_argument("--seed", type=int, default=1)
    cmd_args = parser.parse_args()
    return cmd_args


args = parse_args()

scheduler = EulerDiscreteScheduler.from_pretrained(args.model_id, subfolder="scheduler")
pipe = StableDiffusionPipeline.from_pretrained(
    args.model_id,
    scheduler=scheduler,
    use_auth_token=True,
    revision="fp16",
    variant="fp16",
    torch_dtype=torch.float16,
    safety_checker=None,
)
pipe = pipe.to("cuda")

pipe.unet = oneflow_compile(pipe.unet)

prompt = args.prompt
with flow.autocast("cuda"):
    for _ in range(args.warmup):
        torch.manual_seed(args.seed)
        images = pipe(
            prompt, height=args.height, width=args.width, num_inference_steps=args.steps
        ).images

    torch.manual_seed(args.seed)
    images = pipe(
        prompt, height=args.height, width=args.width, num_inference_steps=args.steps
    ).images
    for i, image in enumerate(images):
        image.save(f"{prompt}-of-{i}.png")



from onediff.infer_compiler.backends.oneflow.param_utils import (
    update_graph_with_constant_folding_info,
)

from .lora import (
    delete_adapters,
    get_active_adapters,
    load_and_fuse_lora,
    set_and_fuse_adapters,
    unfuse_lora,
)



from pathlib import Path

import torch
from diffusers import DiffusionPipeline
from onediff.infer_compiler import oneflow_compile
from onediff.torch_utils import TensorInplaceAssign

try:
    from onediffx.lora import (
        load_and_fuse_lora,
        unfuse_lora,
        update_graph_with_constant_folding_info,
    )
except ImportError:
    raise RuntimeError(
        "OneDiff onediffx is not installed. Please check onediff_diffusers_extensions/README.md to install onediffx."
    )

MODEL_ID = "stabilityai/stable-diffusion-xl-base-1.0"
pipe = DiffusionPipeline.from_pretrained(
    MODEL_ID, variant="fp16", torch_dtype=torch.float16
).to("cuda")
LORA_MODEL_ID = "hf-internal-testing/sdxl-1.0-lora"
LORA_FILENAME = "sd_xl_offset_example-lora_1.0.safetensors"

pipe.unet = oneflow_compile(pipe.unet)
latents = torch.randn(
    1,
    4,
    128,
    128,
    generator=torch.cuda.manual_seed(0),
    dtype=torch.float16,
    device="cuda",
)

# There are three methods to load LoRA into OneDiff compiled model
# 1. pipe.load_lora_weights (Low Performence)
# 2. pipe.load_lora_weights + TensorInplaceAssign + pipe.fuse_lora (Deprecated)
# 3. onediff.utils.load_and_fuse_lora (RECOMMENDED)


# 1. pipe.load_lora_weights (Low Performence)
# use load_lora_weights without fuse_lora is not recommended,
# due to the disruption of attention optimization, the inference speed is slowed down
pipe.load_lora_weights(LORA_MODEL_ID, weight_name=LORA_FILENAME)
images_fusion = pipe(
    "masterpiece, best quality, mountain",
    generator=torch.manual_seed(0),
    height=1024,
    width=1024,
    num_inference_steps=30,
    latents=latents,
).images[0]
images_fusion.save("test_sdxl_lora_method1.png")
pipe.unload_lora_weights()


# need to rebuild UNet because method 1 has different computer graph with naive UNet
pipe = DiffusionPipeline.from_pretrained(
    MODEL_ID, variant="fp16", torch_dtype=torch.float16
).to("cuda")
pipe.unet = oneflow_compile(pipe.unet)
images_fusion = pipe(
    "masterpiece, best quality, mountain",
    generator=torch.manual_seed(0),
    height=1024,
    width=1024,
    num_inference_steps=30,
).images[0]


# 2. pipe.load_lora_weights + TensorInplaceAssign + pipe.fuse_lora (Deprecated)
# The 'fuse_lora' API is not available in diffuser versions prior to 0.21.0.
pipe.load_lora_weights(LORA_MODEL_ID, weight_name=LORA_FILENAME)
if hasattr(pipe, "fuse_lora"):
    # TensorInplaceAssign is DEPRECATED and NOT RECOMMENDED, please use onediff.utils.load_and_fuse_lora
    with TensorInplaceAssign(pipe.unet):
        pipe.fuse_lora(lora_scale=1.0)
    # The function has to be invoked manually because of the constant folding optimization
    # If you are using OneDiffx instead of diffusers to load LoRA, the function is not necessary
    update_graph_with_constant_folding_info(pipe.unet)
    images_fusion = pipe(
        "masterpiece, best quality, mountain",
        generator=torch.manual_seed(0),
        height=1024,
        width=1024,
        num_inference_steps=30,
        latents=latents,
    ).images[0]
    images_fusion.save("test_sdxl_lora_method2.png")

    with TensorInplaceAssign(pipe.unet):
        pipe.unfuse_lora()
pipe.unload_lora_weights()


# 3. onediff.utils.load_and_fuse_lora (RECOMMENDED)
# load_and_fuse_lora is equivalent to load_lora_weights + fuse_lora
load_and_fuse_lora(pipe, LORA_MODEL_ID, weight_name=LORA_FILENAME, lora_scale=1.0)
images_fusion = pipe(
    "masterpiece, best quality, mountain",
    generator=torch.manual_seed(0),
    height=1024,
    width=1024,
    num_inference_steps=30,
    latents=latents,
).images[0]
unfuse_lora(pipe)
images_fusion.save("test_sdxl_lora_method3.png")


# 4. unfuse_lora can uninstall LoRA weights and restore the weights of UNet
images_fusion = pipe(
    "masterpiece, best quality, mountain",
    generator=torch.manual_seed(0),
    height=1024,
    width=1024,
    num_inference_steps=30,
    latents=latents,
).images[0]
images_fusion.save("test_sdxl_lora_without_lora.png")



import random
from collections import defaultdict
from pathlib import Path
from typing import Dict, List, Tuple

import numpy as np
import pytest
import safetensors.torch

import torch
from diffusers import DiffusionPipeline
from onediff.infer_compiler import oneflow_compile

from onediffx.lora import (
    delete_adapters,
    get_active_adapters,
    load_and_fuse_lora,
    set_and_fuse_adapters,
    unfuse_lora,
)
from PIL import Image
from skimage.metrics import structural_similarity
from torch import Tensor

HEIGHT = 1024
WIDTH = 1024
NUM_STEPS = 30
LORA_SCALE = 0.5
LATENTS = torch.randn(
    1,
    4,
    128,
    128,
    generator=torch.cuda.manual_seed(0),
    dtype=torch.float16,
    device="cuda",
)

image_file_prefix = "/share_nfs/onediff_ci/diffusers/images/1.0"


@pytest.fixture
def prepare_loras() -> Dict[str, Dict[str, Tensor]]:
    loras = [
        "/share_nfs/onediff_ci/diffusers/loras/SDXL-Emoji-Lora-r4.safetensors",
        "/share_nfs/onediff_ci/diffusers/loras/sdxl_metal_lora.safetensors",
        "/share_nfs/onediff_ci/diffusers/loras/simple_drawing_xl_b1-000012.safetensors",
        "/share_nfs/onediff_ci/diffusers/loras/texta.safetensors",
        "/share_nfs/onediff_ci/diffusers/loras/watercolor_v1_sdxl_lora.safetensors",
    ]
    loras = {x: safetensors.torch.load_file(x) for x in loras}
    return loras


@pytest.fixture
def get_loras(prepare_loras) -> Dict[str, Dict[str, Tensor]]:
    def _get_loras():
        return {name: lora_dict.copy() for name, lora_dict in prepare_loras.items()}

    return _get_loras


@pytest.fixture
def get_multi_loras(prepare_loras) -> Dict[str, Dict[str, Tensor]]:
    def _get_multi_loras():
        multi_lora = {}
        current_name = []
        current_lora = []
        for name, lora_dict in prepare_loras.items():
            current_name.append(name)
            current_lora.append(lora_dict)
            multi_lora[tuple(current_name)] = current_lora
        return multi_lora

    return _get_multi_loras


@pytest.fixture
def pipe():
    MODEL_ID = "/share_nfs/hf_models/stable-diffusion-xl-base-1.0"
    pipeline = DiffusionPipeline.from_pretrained(
        MODEL_ID, variant="fp16", torch_dtype=torch.float16
    ).to("cuda")
    return pipeline


def generate_image(pipe):
    image = pipe(
        "masterpiece, best quality, mountain",
        height=HEIGHT,
        width=WIDTH,
        num_inference_steps=NUM_STEPS,
        generator=torch.manual_seed(0),
        latents=LATENTS.clone(),
    ).images[0]
    return image


def prepare_target_images(pipe, loras):
    target_images_list = [
        f"{image_file_prefix}/test_sdxl_lora_{str(Path(name).stem)}_{HEIGHT}_{WIDTH}.png"
        for name in loras
    ]
    if all(Path(x).exists() for x in target_images_list):
        return

    print("Didn't find target images, try to generate...")
    for name, lora in loras.items():
        pipe.load_lora_weights(lora.copy())
        pipe.fuse_lora()
        image = generate_image(pipe)
        pipe.unfuse_lora()
        pipe.unload_lora_weights()
        image.save(
            f"{image_file_prefix}/test_sdxl_lora_{str(Path(name).stem)}_{HEIGHT}_{WIDTH}.png"
        )
    torch.cuda.empty_cache()


def prepare_target_images_multi_lora(pipe, loras, multi_loras):
    target_images_list = [
        f"{image_file_prefix}/test_sdxl_multi_lora_{'_'.join([str(Path(name).stem) for name in names])}_{HEIGHT}_{WIDTH}.png"
        for names in multi_loras
    ]
    if all(Path(x).exists() for x in target_images_list):
        return

    for name, lora in loras.items():
        print(f"loading name: {name}")
        pipe.load_lora_weights(lora.copy(), adapter_name=str(Path(name).stem))

    print("Didn't find target images, try to generate...")
    for names, loras in multi_loras.items():
        names = [str(Path(name).stem) for name in names]
        pipe.set_adapters(
            names,
            [
                LORA_SCALE,
            ]
            * len(names),
        )
        image = generate_image(pipe)
        image_name = f"{image_file_prefix}/test_sdxl_multi_lora_{'_'.join([str(Path(name).stem) for name in names])}_{HEIGHT}_{WIDTH}.png"
        image.save(image_name)
    pipe.unload_lora_weights()
    torch.cuda.empty_cache()


def preload_multi_loras(pipe, loras):
    for name, lora in loras.items():
        load_and_fuse_lora(
            pipe,
            lora.copy(),
            adapter_name=Path(name).stem,
        )
        unfuse_lora(pipe)


def test_lora_loading(pipe, get_loras):
    pipe.unet = oneflow_compile(pipe.unet)
    pipe(
        "a cat",
        height=HEIGHT,
        width=WIDTH,
        num_inference_steps=NUM_STEPS,
    ).images[0]
    loras = get_loras()
    prepare_target_images(pipe, loras)

    for name, lora in loras.items():
        load_and_fuse_lora(pipe, lora.copy())
        images_fusion = generate_image(pipe)
        target_image = np.array(
            Image.open(
                f"{image_file_prefix}/test_sdxl_lora_{str(Path(name).stem)}_{HEIGHT}_{WIDTH}.png"
            )
        )
        curr_image = np.array(images_fusion)
        ssim = structural_similarity(
            curr_image, target_image, channel_axis=-1, data_range=255
        )
        unfuse_lora(pipe)
        images_fusion.save(
            f"./test_sdxl_lora_{str(Path(name).stem)}_{HEIGHT}_{WIDTH}.png"
        )
        print(f"lora {name} ssim {ssim}")
        assert ssim > 0.92, f"LoRA {name} ssim too low"


def test_multi_lora_loading(pipe, get_multi_loras, get_loras):
    pipe.unet = oneflow_compile(pipe.unet)
    multi_loras = get_multi_loras()
    loras = get_loras()
    prepare_target_images_multi_lora(pipe, loras, multi_loras)
    preload_multi_loras(pipe, loras)

    for names, loras in multi_loras.items():
        names = [str(Path(name).stem) for name in names]
        set_and_fuse_adapters(
            pipe,
            names,
            [
                LORA_SCALE,
            ]
            * len(names),
        )

        images_fusion = generate_image(pipe)
        image_name = "_".join([str(Path(name).stem) for name in names])
        target_image = np.array(
            Image.open(
                f"{image_file_prefix}/test_sdxl_multi_lora_{image_name}_{HEIGHT}_{WIDTH}.png"
            )
        )
        images_fusion.save(f"./test_sdxl_multi_lora_{image_name}_{HEIGHT}_{WIDTH}.png")
        curr_image = np.array(images_fusion)
        ssim = structural_similarity(
            curr_image, target_image, channel_axis=-1, data_range=255
        )
        print(f"lora {names} ssim {ssim}")
        assert ssim > 0.92, f"LoRA {names} ssim too low"


def test_get_active_adapters(pipe, get_multi_loras):
    multi_loras = get_multi_loras()
    for names, _ in multi_loras.items():
        names = [str(Path(name).stem) for name in names]
        set_and_fuse_adapters(pipe, names)
        active_adapters = get_active_adapters(pipe)
        print(f"current adapters: {active_adapters}, target adapters: {names}")
        assert set(active_adapters) == set(names)


def test_delete_adapters(pipe, get_multi_loras):
    multi_loras = get_multi_loras()
    for names, _ in multi_loras.items():
        names = [str(Path(name).stem) for name in names]
        names_to_delete = random.sample(names, k=random.randint(0, len(names)))
        set_and_fuse_adapters(pipe, names)
        delete_adapters(pipe, names_to_delete)
        active_adapters = get_active_adapters(pipe)
        print(
            f"current adapters: {active_adapters}, target adapters: {list(set(names) - set(names_to_delete))}"
        )
        assert set(active_adapters) == set(names) - set(names_to_delete)



import argparse
import json
import time

import torch

from diffusers import DPMSolverMultistepScheduler, KolorsPipeline
from onediff.infer_compiler import oneflow_compile
from onediffx import compile_pipe, load_pipe, quantize_pipe, save_pipe


def parse_args():
    parser = argparse.ArgumentParser(
        description="Use onediif to accelerate image generation with Kolors"
    )
    parser.add_argument(
        "--model",
        type=str,
        default="Kwai-Kolors/Kolors-diffusers",
        help="Model path or identifier.",
    )
    parser.add_argument(
        "--compiler",
        type=str,
        default="none",
        help="Compiler backend to use. Options: 'none', 'nexfort', 'oneflow'",
    )
    parser.add_argument(
        "--compiler-config", type=str, help="JSON string for compiler config."
    )
    parser.add_argument(
        "--quantize-config", type=str, help="JSON string for quantization config."
    )
    parser.add_argument(
        "--prompt",
        type=str,
        default='一张瓢虫的照片，微距，变焦，高质量，电影，拿着一个牌子，写着"可图"',
        help="Prompt for the image generation.",
    )
    parser.add_argument(
        "--height", type=int, default=1024, help="Height of the generated image."
    )
    parser.add_argument(
        "--width", type=int, default=1024, help="Width of the generated image."
    )
    parser.add_argument(
        "--guidance_scale",
        type=float,
        default=6.5,
        help="The scale factor for the guidance.",
    )
    parser.add_argument(
        "--num-inference-steps", type=int, default=50, help="Number of inference steps."
    )
    parser.add_argument(
        "--saved-image",
        type=str,
        default="./kolors.png",
        help="Path to save the generated image.",
    )
    parser.add_argument(
        "--seed", type=int, default=66, help="Seed for random number generation."
    )
    parser.add_argument(
        "--warmup-iterations",
        type=int,
        default=1,
        help="Number of warm-up iterations before actual inference.",
    )
    parser.add_argument(
        "--run_multiple_resolutions",
        type=(lambda x: str(x).lower() in ["true", "1", "yes"]),
        default=False,
    )
    parser.add_argument("--oneflow_save", action=argparse.BooleanOptionalAction)
    parser.add_argument("--oneflow_load", action=argparse.BooleanOptionalAction)
    return parser.parse_args()


args = parse_args()

device = torch.device("cuda")


class KolorsGenerator:
    def __init__(
        self, model, compiler_config=None, quantize_config=None, compiler="none"
    ):
        self.pipe = KolorsPipeline.from_pretrained(
            model, torch_dtype=torch.float16, variant="fp16"
        ).to(device)
        self.pipe.scheduler = DPMSolverMultistepScheduler.from_config(
            self.pipe.scheduler.config, use_karras_sigmas=True
        )

        if compiler == "nexfort":
            if compiler_config:
                print("nexfort backend compile...")
                self.pipe = self.compile_pipe(self.pipe, compiler_config)

            if quantize_config:
                print("nexfort backend quant...")
                self.pipe = self.quantize_pipe(self.pipe, quantize_config)
        elif compiler == "oneflow":
            print("oneflow backend compile...")
            # self.pipe.unet = self.oneflow_compile(self.pipe.unet)
            self.pipe = compile_pipe(self.pipe, ignores=["text_encoder", "vae"])

    def warmup(self, gen_args, warmup_iterations):
        warmup_args = gen_args.copy()

        warmup_args["generator"] = torch.Generator(device=device).manual_seed(0)

        print("Starting warmup...")
        start_time = time.time()
        if args.oneflow_load:
            load_pipe(self.pipe, dir="cached_pipe")

        for _ in range(warmup_iterations):
            self.pipe(**warmup_args)

        if args.oneflow_save:
            save_pipe(self.pipe, dir="cached_pipe")
        end_time = time.time()
        print("Warmup complete.")
        print(f"Warmup time: {end_time - start_time:.2f} seconds")

    def generate(self, gen_args):
        gen_args["generator"] = torch.Generator(device=device).manual_seed(args.seed)

        # Run the model
        start_time = time.time()
        images = self.pipe(**gen_args).images
        end_time = time.time()

        images[0].save(args.saved_image)

        return images[0], end_time - start_time

    def compile_pipe(self, pipe, compiler_config):
        options = compiler_config
        pipe = compile_pipe(
            pipe, backend="nexfort", options=options, fuse_qkv_projections=True
        )
        return pipe

    def quantize_pipe(self, pipe, quantize_config):
        pipe = quantize_pipe(pipe, ignores=[], **quantize_config)
        return pipe

    def oneflow_compile(self, unet):
        return oneflow_compile(unet)


def main():
    nexfort_compiler_config = (
        json.loads(args.compiler_config) if args.compiler_config else None
    )
    nexfort_quantize_config = (
        json.loads(args.quantize_config) if args.quantize_config else None
    )

    kolors = KolorsGenerator(
        args.model,
        nexfort_compiler_config,
        nexfort_quantize_config,
        compiler=args.compiler,
    )

    gen_args = {
        "prompt": args.prompt,
        "num_inference_steps": args.num_inference_steps,
        "height": args.height,
        "width": args.width,
        "guidance_scale": args.guidance_scale,
    }

    kolors.warmup(gen_args, args.warmup_iterations)

    image, inference_time = kolors.generate(gen_args)
    print(
        f"Generated image saved to {args.saved_image} in {inference_time:.2f} seconds."
    )
    cuda_mem_after_used = torch.cuda.max_memory_allocated() / (1024**3)
    print(f"Max used CUDA memory : {cuda_mem_after_used:.3f}GiB")

    if args.run_multiple_resolutions:
        print("Test run with multiple resolutions...")
        sizes = [1024, 768, 576, 512, 256]
        for h in sizes:
            for w in sizes:
                gen_args["height"] = h
                gen_args["width"] = w
                print(f"Running at resolution: {h}x{w}")
                start_time = time.time()
                kolors.generate(gen_args)
                end_time = time.time()
                print(f"Inference time: {end_time - start_time:.2f} seconds")
                assert (
                    end_time - start_time
                ) < 20, "Resolution switch test took too long"


if __name__ == "__main__":
    main()



import os

from setuptools import find_packages, setup


local_scheme = os.getenv("VERSION_LOCAL_SCHEME", "node-and-date")

setup(
    name="onediffx",
    description="onediff extensions for diffusers",
    url="https://github.com/siliconflow/onediff",
    author="OneDiff contributors",
    license="Apache-2.0",
    author_email="contact@siliconflow.com",
    packages=find_packages(),
    python_requires=">=3.7.0",
    install_requires=[
        "transformers>=4.27.1",
        "diffusers>=0.19.3",
        "accelerate",
        "torch",
        "onefx",
        "omegaconf",
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    use_scm_version={
        "root": "../",
        "version_file": "./onediffx/_version.py",
        "fallback_version": "0.0.0",
        "version_scheme": "guess-next-dev",
        "local_scheme": local_scheme,
    },
    setup_requires=["setuptools_scm"],
)



from onediff.infer_compiler import OneflowCompileOptions

from .compilers.diffusion_pipeline_compiler import (
    compile_pipe,
    load_pipe,
    quantize_pipe,
    save_pipe,
)

try:
    from ._version import version as __version__, version_tuple
except ImportError:
    __version__ = "unknown version"
    version_tuple = (0, 0, "unknown version")

__all__ = [
    "compile_pipe",
    "save_pipe",
    "load_pipe",
    "OneflowCompileOptions",
    "quantize_pipe",
]
